#!/bin/bash
#PBS -N starrunalldata2-completedata
#PBS -l select=1:ncpus=24:mem=100gb:scratch_local=1024gb
#PBS -j oe
#PBS -m e
#PBS -l walltime=20:00:00

#The first draft of the script was created by P. Ryšavý, the supervisor of this thesis, further modifications were made by H. Mertanová.

trap 'clean_scratch' TERM EXIT

# get info about the machine
cat /proc/cpuinfo | grep '^model name' | sort -u

# runs STAR aligner to align the seqences

cd $SCRATCHDIR || exit 1
DATADIR=/storage/plzen1/home/mertahan/scc/ #change to your path to directory containing data directory (named data)
					   #the results will be also saved here
FAREDIR=/storage/plzen1/home/mertahan/fa/ #change to your path to reference directory (named ref) containing (.fa.gz and .gtf)

# copy all the data to scratch
cp -r $FAREDIR/ref $SCRATCHDIR || exit 1
mkdir data || exit 1
cp $DATADIR/data/* ./data || exit 1

cd data || exit 1
mkdir ../starout || exit 1

for f in *_R1_???.fastq.gz; do
    echo "Processing $f file..";
    # replace the read name
    fastQ2="${f/\_R1\_/\_R2\_}"
    # output file name
    output="${f/\_R1\_/_}"
    output="${output/fastq\.gz/bam}"
    samplename=$(echo $output | cut -f1 -d.)


  if [ ! -s ../starout/$samplename/Aligned.sortedByCoord.out.bam ]; then
    echo `date` $f $fastQ2;
    mkdir -p ../starout/$samplename;
     /storage/plzen1/home/mertahan/progs/STAR --runThreadN 24 --genomeDir ../ref \
          --sjdbGTFfile ../ref/Homo_sapiens.GRCh38.86.gtf --sjdbOverhang 125 \
          --readFilesIn $f $fastQ2 \
          --readFilesCommand zcat \
          --outSAMtype BAM SortedByCoordinate \
          --outFileNamePrefix ../starout/$samplename/ \
          --outTmpDir tmp
  fi;
done; date

cd $SCRATCHDIR

# now copy everything to the storage
cp -r starout $DATADIR || exit 1
rm -r *
