"""
3.step

generates k-mer clusters from the table with k-mer counts in all input files

Parameters : –inptable (input table name) 
         –eps (number)
Input : table name from output of the second step, 
        eps - parameter of DBSCAN
Output : clusters.txt, each line containing k-mers in one cluster, lines sorted by size,
        graph showing some of the resulting clusters after PCA-dimension reduction,
        sizes.txt containing information about the size of clusters,
        plot.pkl containing data necessary to generate graph

"""


from sklearn.cluster import DBSCAN
import numpy as np
import pickle


from sklearn.cluster import AgglomerativeClustering
import random
import matplotlib
import matplotlib as mpl
mpl.use('Agg')
from matplotlib import pyplot as plt

from sklearn.decomposition import PCA

import argparse
parser = argparse.ArgumentParser()
parser.add_argument("--inptable", default='table.txt', help="echo the string you use here")
parser.add_argument("--eps", default=10, help="echo the string you use here")
args = parser.parse_args()


local = False
remOut = True #removes outliers from the graph
sigNo = 50

if (local):
    table = 'table0.txt'
    percentage = 1
    eps = 3
else:
    table = args.inptable
    percentage = 0.1 # sampling 1 = 100%
    eps = float(args.eps)


def isOut(x, mean, sd):
    return x < mean - 3*sd or x > mean + 3*sd


# reading file

kmersLabels = []
kmers = []

f = open(table, "r")

f.readline()  # column names
f.readline()  # blank line

while True:
    try:
        line = f.readline().rstrip("\n").split(" ", 1)
        kmersLabels.append(line[0])  # kmer name
        kmers.append(eval(line[1]))  # kmer counts
    except:
        break

f.close()

X = np.array(kmers)
kmersLabels = np.array(kmersLabels[:-1:])  # last label is blank line

# sampling
samplesNo = round(len(X) * percentage)

idxs = random.sample(range(len(X)), samplesNo)
X = X[idxs]
kmersLabels = kmersLabels[idxs]


#performing DBSCAN
clst = DBSCAN(eps=eps, min_samples=2)
clst.fit(X)

clusters = []

numberClusters = len(set(clst.labels_)) - (1 if -1 in clst.labels_ else 0)

#counting occurences in every cluster 
ldict = {}
for lab in clst.labels_:
    if lab in ldict:
        ldict[lab] += 1
    else:
        ldict[lab] = 1

#sorting clusters by size
s = sorted(ldict.items(), key=lambda  kv: kv[1], reverse=True)
labelsSorted = [s[i][0] for i in range(len(s))] #vsetky labely zoradene od najvacsieho po najmensi

for i in labelsSorted:
    if (i == -1): #let out not clustered
        continue
    cl_kmers = kmersLabels[clst.labels_ == i]
    clusters.append(cl_kmers.tolist())

#saving results
f = open("clusters.txt", "w")
for cl in clusters:
    f.write(" ".join(cl) + "\n")
f.close()

#printing sizes of clusters

clustersSizes = [s[i][1] for i in range(len(ldict))]
g = open('sizes.txt', 'w')
for size in clustersSizes:
    g.write(str(size) + '\n')
g.close()



## plot
plotdata = [X, clst.labels_]

tmpfile = open('plot.pkl', 'wb')
pickle.dump(plotdata, tmpfile)
tmpfile.close()


#pickle X, clst.labels_,
#functions isOut,
print("number of clusters: " + str(len(clusters)))

#colors = ['r', 'g', 'b', 'y', 'm', 'lawngreen', 'aqua', 'navy', ] + ['chartreuse', 'darkslategrey', 'lavender', 'floralwhite', 'powderblue', 'deepskyblue', 'violet', 'skyblue', 'peru', 'gainsboro', 'dodgerblue', 'darkblue', 'mediumpurple', 'darkgray', 'lightsteelblue', 'lavenderblush', 'turquoise', 'darkslateblue', 'forestgreen', 'steelblue', 'blanchedalmond', 'azure', 'saddlebrown', 'lightgreen', 'orange', 'springgreen', 'indianred', 'crimson', 'darkgreen', 'mediumslateblue', 'mediumseagreen', 'grey', 'brown', 'magenta', 'mediumaquamarine', 'pink', 'orangered', 'lightcoral', 'lightgoldenrodyellow', 'aquamarine', 'darkorange', 'lightseagreen', 'dimgrey', 'sandybrown', 'lightpink', 'gray', 'yellowgreen', 'lightgrey', 'papayawhip', 'rosybrown', 'gold', 'olive', 'white', 'orchid', 'teal', 'lightblue', 'mediumblue', 'plum', 'darkcyan', 'mistyrose', 'darkviolet', 'lightgray', 'mintcream', 'dimgray', 'greenyellow', 'paleturquoise', 'darkgoldenrod', 'tan', 'lightskyblue', 'navy', 'silver', 'goldenrod', 'peachpuff', 'ghostwhite', 'salmon', 'palevioletred', 'beige', 'indigo', 'lightyellow', 'cornflowerblue', 'linen', 'lightsalmon', 'slategrey', 'cyan', 'lightslategrey', 'royalblue', 'palegreen', 'seagreen', 'darkseagreen', 'slategray', 'lawngreen', 'lemonchiffon', 'lightslategray', 'seashell', 'khaki', 'mediumvioletred', 'olivedrab', 'moccasin', 'mediumorchid', 'midnightblue', 'darkgrey', 'darkolivegreen', 'black', 'honeydew', 'maroon', 'lime', 'darkslategray', 'whitesmoke', 'bisque', 'burlywood', 'darkred', 'slateblue', 'darkturquoise', 'limegreen', 'purple', 'darkorchid', 'thistle', 'aliceblue', 'wheat', 'rebeccapurple', 'darksalmon', 'mediumturquoise', 'hotpink', 'cadetblue', 'ivory', 'cornsilk', 'lightcyan', 'blueviolet', 'firebrick', 'navajowhite', 'antiquewhite', 'sienna', 'mediumspringgreen', 'chocolate', 'snow', 'oldlace', 'fuchsia', 'aqua', 'yellow', 'coral', 'deeppink', 'palegoldenrod', 'tomato', 'darkkhaki', 'darkmagenta']
#colors = 1000 * colors
colors = ['black' , 'red', 'yellow', 'purple', 'violet', 'blue', 'brown', 'green', 'aqua', 'orange', 'lime'] #+ colors
colrs = [colors[i] for i in range(10)]

## 2D reduction
pca = PCA(n_components=2)

#print(X)

X_pca = pca.fit_transform(X)

if (remOut):
    # removing outliers
    xmean = np.mean(X_pca[:, 0])
    xsd = np.std(X_pca[:, 0])
    ymean = np.mean(X_pca[:, 1])
    ysd = np.std(X_pca[:, 1])
    x = [vec[0] for vec in X_pca if (not isOut(vec[0], xmean, xsd) and not isOut(vec[1], ymean, ysd))]
    y = [vec[1] for vec in X_pca if (not isOut(vec[0], xmean, xsd) and not isOut(vec[1], ymean, ysd))]
    b = [(not isOut(vec[0], xmean, xsd) and not isOut(vec[1], ymean, ysd)) for vec in X_pca]
    label = clst.labels_[b]



    # printing only 10 clusters (not first 10 largest)
    if (not local):
        ldict = {}
        for lab in label:
            if lab in ldict:
                ldict[lab] += 1
            else:
                ldict[lab] = 1

        s = sorted(ldict.items(), key=lambda  kv: kv[1], reverse=True)
        take10 = [s[i][0] for i in range(10, 20)] #10 labelov
        my10 = [i in take10 for i in label]
        x = [x[i] for i in range(0, len(my10)) if my10[i]]
        y = [y[i] for i in range(0, len(my10)) if my10[i]]
        label = label[my10]






else:
    x = [vec[0] for vec in X_pca]
    y = [vec[1] for vec in X_pca]
    label = clst.labels_



fig = plt.figure()
plt.title("percent " + str(percentage*100))
plt.scatter(x, y, c=label, cmap=matplotlib.colors.ListedColormap(colrs))
plt.savefig("PCA_2D_clusters.png")
#plt.show()

