"""
5.step

compares our approach with the reference-based solution (STAR output)

params: –refclust (the name of folder containing ref-membership-tables)
        –myclust (the name of folder containing our-membership-tables)
        –ressig (the name of folder containing the Important cathegories)
input : folders from params
output: results2.txt (containing error for each subject from the dataset)

"""


import random
import pickle
import os
import argparse
import math
parser = argparse.ArgumentParser()
parser.add_argument("--refclust", default='Resultsref', help="echo the string you use here")
parser.add_argument("--myclust", default='ResultsC', help="echo the string you use here")
parser.add_argument("--ressig", default='Ressig', help="echo the string you use here")
args = parser.parse_args()
#print(args.genome)

percent = 0.03

def toStr(err):
    try:
        r = str(err.data[0])
    except:
        r = str(err)
    return r

def toFloat(csr):
    try:
        r = float(csr.data[0])
    except:
        r = float(csr)
    return r


def loadsignif(filename):
    signif = []
    f = open(filename, 'r')
    signif = f.readline().split()
    return signif


#to accept dict.keys or array of important reads
def ranCup(struct, percent, no=-1):
    k = list(struct)
    elno = len(struct)
    if (no == -1):
        cupno = round(elno*(elno-1)/2 * (percent/100 * percent/100))
    else:
        cupno = no
    idxs = random.choices(range(len(struct)), k=cupno)
    idxs2 = random.choices(range(len(struct)), k=cupno)
    sam = [k[i] for i in idxs]
    sam2 = [k[i] for i in idxs2]

    cuples = [(r1,r2) for r1, r2 in zip(sam, sam2) if r1 != r2]

    while (len(cuples) < cupno):
        missing = cupno - len(cuples)
        idxs = random.choices(range(len(struct)), k=missing)
        idxs2 = random.choices(range(len(struct)), k=missing)
        sam = [k[i] for i in idxs]
        sam2 = [k[i] for i in idxs2]
        cuples = cuples + [(r1, r2) for r1, r2 in zip(sam, sam2) if r1 != r2]

    return cuples

def ransam(data, percent=100, no=0):
    if (no == 0):
        no = round(len(data) * percent/100)
    idxs = random.choices(range(len(data)), k=no)
    return [data[i] for i in idxs]

local = False


from scipy.sparse import  csr_matrix

def metric(diff, squares=True, absol=False):
    if squares:
        return diff*diff
    if absol:
        return abs(diff)

if (local):
    myClustDir = ''
    refDir = ''
    ressigdir = ''
else:
    myClustDir = args.myclust + '/'
    refDir = args.refclust + '/'
    ressigdir = args.ressig + '/'

fres = open('results2.txt', "w")

#loading files
signames = sorted([ressigdir + fname for fname in os.listdir(ressigdir)])
refNames = sorted([refDir + fname for fname in os.listdir(refDir)])
myNames = sorted([myClustDir + fname for fname in os.listdir(myClustDir)])

fixedNo = -1

for sname, rname, mname in zip(signames, refNames, myNames):
    signifname = sname
    refname = rname
    myname = mname

    file2 = open(myname, 'rb')
    myTable = pickle.load(file2)
    file2.close()
    file2 = open(refname, 'rb')
    refTable = pickle.load(file2)
    file2.close()

    print('doneLoadingComparing' + myname)

#overall error random cuples
    error = 0
    error11 = 0
    normal1 = 1
    cuples = ranCup(refTable.keys(), percent, fixedNo)
    no = len(cuples)
    fixedNo = no

    print('doneCouples')

    for read1, read2 in cuples:
        myprod = 0
        refprod = 0
        if (read1 in myTable and read2 in myTable):
            val1 = myTable[read1]
            val2 = myTable[read2]
            myprod = val1.dot(val2.transpose())
        if (read1 in refTable and read2 in refTable):
            val1 = refTable[read1]
            val2 = refTable[read2]
            refprod = val1.dot(val2.transpose())

        error += metric(myprod-refprod, squares=True)

        if (read1 in myTable and read2 in myTable):
            normal1 += 1
            error11 += metric(myprod-refprod, squares=True)


    #error only on significant reads
    error2 = 0
    normal2 = 1
    if (True):
        signif = loadsignif(signifname)
        print('done signif loading')
        cuples2 = ranCup(signif, 0.01, no)

        for read1, read2 in cuples2:
            myprod = 0
            refprod = 0
            if (read1 in myTable and read2 in myTable):
                normal2 += 1
                val1 = myTable[read1]
                val2 = myTable[read2]
                myprod = val1.dot(val2.transpose())
            if (read1 in refTable and read2 in refTable):
                val1 = refTable[read1]
                val2 = refTable[read2]
                refprod = val1.dot(val2.transpose())

            error2 += metric(myprod - refprod, squares=True)
    error3 = 0
    error4 = 0
    if (True):

        vec = next(iter(refTable.values()))
        vsize = vec.shape[1]

        lookup = dict()
        for i in signif:
            lookup[i] = 0

        sl = [0 for _ in range(vsize)]
        nor = [0 for _ in range(vsize)]
        numNS = 0 # number of not significant reads
        numSall = 0
        numS = 0

        for r, vec in refTable.items():
            if (r not in lookup):  # read is not signif
                numNS += 1
                idxs = vec.indices
                vals = vec.data
                for idx, val in zip(idxs, vals):
                    sl[idx] += float(val)
                    if (val != 0): # normalize by the number of signif reads marked unsignif for that cluster
                        nor[idx] += 1
            else:
                numSall += 1

        sigsam = ransam(signif, no=no)

        for sr in sigsam:
            if (sr in refTable):
                numS += 1
                vec = refTable[sr]
                idxs = vec.indices
                vals = vec.data
                for idx, val in zip(idxs, vals):
                    #error 3
                    if (nor[idx] != 0):
                        error3 += sl[idx] / nor[idx] * val
                    #error 4
                    if (numNS != 0):
                        error4 += sl[idx] / numNS * val


    errorPrev1 = error/fixedNo
    errorPrev2 = error2/fixedNo



    error = error / normal1
    error2 = error2 / normal2

    error11 = error11/normal1


    if (numS != 0):
        error3 = error3 / numS
        error4 = error4 / numS

    fres.write(myname + '  ' + toStr(errorPrev1) + ' ' + toStr(errorPrev2) + ' '
               + str(error3) + ' ' + str(error4)
               + ' ' + str(numS) + ' ' + str(numSall) + ' ' + str(numNS) + ' ' + str(fixedNo)
               + ' ' + toStr(error) + ' ' + toStr(error2) + ' '
               + str(normal1) + ' ' + str(normal2) + ' ' + toStr(error11) + '\n')

fres.close()



