"""
4.step

clusters reads into the clusters according to k-mer clusters
recognizes the important cathegory

Parameters : –ksize (the length of k-mers) 
             –dbtable (result table name)
             –datadir (the folder containing data as described in 8.1)
Input : clusters.txt from the previous step,
         original data folder with two files for every subject
Output : membership-table for every subject in data
         Ressig folder containinf ID-s of reads in Important cathegory for every subject

"""


import sys
from Bio import SeqIO
from Bio.SeqIO import FastaIO
import gzip
import numpy as np
import itertools
import os
import pickle

from scipy.sparse import csr_matrix

import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--ksize", default=25, help="echo the string you use here")
parser.add_argument("--datadir", default='data/')
parser.add_argument("--dbtable", default='clusters.txt')
args = parser.parse_args()

kmersize = int(args.ksize)

local = False
sigrange = 50

sigreads = []

def getKmers(seq, length):
    kmers = []
    for i in range(0, len(seq) - length + 1):
        kmer = seq[i:i + length]
        kmers.append(kmer)
    return kmers


def dict2CSR(clusterIDdict):  # keys are positions and values are data in sparse csr_matrix, returns sparse vector
    row = [0 for i in range(len(clusterIDdict))]
    col = [int(x) for x in clusterIDdict.keys()]
    data = [int(x) for x in clusterIDdict.values()]
    return csr_matrix((data, (row, col)), shape=(1, numClusters))


def readClusterVec(read, hashtable):
    global sigreads
    readsig = False

    clusterIDdict = {}
    numhits = 0

    rc = read.reverse_complement()
    kmers = getKmers(str(read.seq), kmersize) + getKmers(str(rc.seq),
                                                         kmersize)  # all kmers in forward and backward read

    for kmer in kmers:
        if kmer in hashtable:
            numhits += 1
            clst = hashtable[kmer]
            readsig |= clst >= 1 and clst <= sigrange

            if (clusterIDdict.__contains__(clst)):
                clusterIDdict[clst] += 1
            else:
                clusterIDdict[clst] = 1

    if (readsig):
        sigreads.append(read.name)

    return dict2CSR(clusterIDdict)

#creating the name for output
def mergedName(nameR1R2):
    idx = nameR1R2.find("_R1_")
    idx2 = nameR1R2.find("_R2_")
    if (idx != -1):
        return nameR1R2[0:idx]
    if (idx2 != -1):
        return nameR1R2[0:idx2]
    else:
        return "invalidName"


if (local):
    fastqgzPath = 'data2/'  
    clusterFileName = 'clustersDBrows.txt'
else:
    fastqgzPath = args.datadir + '/'
    clusterFileName = 'clusters.txt'

# creating hash table

numClusters = 0

hashtable = {}

f = open(clusterFileName, 'r')

for line in f:
    kmers = line.split()
    for kmer in kmers:
        hashtable[kmer] = numClusters
    if (line != ''):
        numClusters += 1

f.close()

print('hashtable created')
print(numClusters)

# getting all fastq files:

curDir = os.path.dirname(os.path.realpath('__file__'))
dataDirFiles = os.path.join(curDir, fastqgzPath)
files = sorted([fastqgzPath + fname for fname in os.listdir(dataDirFiles)])
resFiles = sorted(os.listdir(dataDirFiles))
print(files)
os.system('mkdir ResultsC')
os.mkdir('Ressig')

# reading fastq file
for i in range(0, len(files) - 1, 2):
    sigreads = []

    resTable = {}

    R1file = gzip.open(files[i], "rt")
    R2file = gzip.open(files[i + 1], "rt")

    readsR1 = SeqIO.parse(R1file, 'fastq')
    readsR2 = SeqIO.parse(R2file, 'fastq')
	
    #creates the sparse vector for each read
    for read1, read2 in zip(readsR1, readsR2):

        readVec1 = readClusterVec(read1, hashtable)
        readVec2 = readClusterVec(read2, hashtable)
        mergedR1R2 = readVec1 + readVec2
        kmerHitsNo = mergedR1R2.sum()

        if (kmerHitsNo != 0):
            relativeVec = mergedR1R2 / kmerHitsNo
            resTable[read1.name] = relativeVec  # asserting each read a sparse vector
        else:
            resTable[read1.name] = mergedR1R2

    # saves cluster table to corresponding .pkl name : should by same as reference aligning
    resfname = mergedName(resFiles[i]) + '.pkl'
    tmpfile = open(resfname, 'wb')
    pickle.dump(resTable, tmpfile)
    tmpfile.close()

    os.system('mv ' + resfname + ' ResultsC')

    resfname = mergedName(resFiles[i]) + '.txt'

    f = open(resfname, 'w')
    f.write(" ".join(sigreads))
    f.close()
    os.system('mv ' + resfname + ' Ressig')

    R1file.close()
    R2file.close()

print('done')
