""" <mapping reads to gene names according to .BAM files and reference .gtf file>
    Copyright (C) <2019>  <Hana Mertanová>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

#the code from  https://htseq.readthedocs.io/en/release_0.9.1/tour.html licenced under GNU GPL
#was used in  lines 49-54, 90-107


"""
from aligned .bam files extracts gene names for each read and then creates the reference table of membership

input : Homo_sapiens.GRCg38.86.gtf.gz
     : starout_directories containing Aligned.sortedByCoord.out.bam
output : creates directory Results2 - containing for every folder corresponding name_readToGene.pkl
       : creates also Results containing not comprimed files
"""



import os
import HTSeq
import pickle

from scipy.sparse import  csr_matrix

import itertools


def dict2CSR(clusterIDdict, numClusters): # keys are positions and values are data in sparse csr_matrix, returns sparse vector
    row = [0 for i in range(len(clusterIDdict))]
    col = [int(x) for x in clusterIDdict.keys()]
    data = [int(x) for x in clusterIDdict.values()]
    return csr_matrix((data, (row, col)), shape=(1, numClusters))

gtf_file = HTSeq.GFF_Reader("Homo_sapiens.GRCh38.86.gtf.gz", end_included=True)
exons = HTSeq.GenomicArrayOfSets("auto", stranded=False)
for feature in gtf_file:
    if feature.type == "exon":
        at = feature.attr
        geneName = at['gene_name']
        exons[feature.iv] += geneName

print("gtfDone")

print([name for name in os.listdir(".") if os.path.isdir(name)])

fols = [os.path.abspath(name) for name in os.listdir(".") if (os.path.isdir(name) and name != 'Results')]

results = []
resultsDict = []


found = 0
nfound = 0


#using output indices of STAR to determine the gene name each read is aligned to
for fol in fols:
    resName = str(fol) + "_readToGene.pkl"
    resfName = str(fol) + "_readToGene"
    results.append(resName)
    resultsDict.append(resfName)

    f = open(resfName, "w") #for writing readName : geneName
    tmpfile = open(resName, 'wb')

    bamname = os.path.join(fol, 'Aligned.sortedByCoord.out.bam')
    bam_file = HTSeq.BAM_Reader(bamname)

    genes = {}
    readsDict = {}
    geneNumber = 0


    resTable = {}

    for alnmt in bam_file:

        if alnmt.aligned:
            iset = None
            for iv2, step_set in exons[alnmt.iv].steps():
                if iset is None:
                    iset = step_set.copy()
                else:
                    iset.intersection_update(step_set)

            if (iset):
                found += 1

                for gen in iset:
                    if gen not in genes:
                        genes[gen] = geneNumber
                        geneNumber += 1


                readsDict[alnmt.read.name] = iset

                f.write(str(alnmt.read.name) + " " + " ".join(iset) + "\n")

            else:
                nfound += 1
    #

    #creating the reference-membership table
    for read, value in readsDict.items():
        clusterIDdict = {}

        for gen in value:
            clst = genes[gen]
            clusterIDdict[clst] = 1

        sparseVec = dict2CSR(clusterIDdict, geneNumber)

        nhit = sparseVec.sum()

        if ( nhit != 0):
            relativeSVec = sparseVec / nhit
            resTable[read] = relativeSVec  # asserting each read a sparse vector
        else:
            resTable[read] = sparseVec

    pickle.dump(resTable, tmpfile)
    tmpfile.close()

    f.close()

os.system('mkdir Results')
for r in results:
    os.system('mv ' + r + ' Results')

os.system('mkdir Results2')

for r in resultsDict:
    os.system('mv ' + r + ' Results2')

print("aligned reads count = " + str(found))
print("not aligned reads count = " + str(nfound))
