#!/bin/bash
#PBS -N runAllPhases
#PBS -l select=1:ncpus=4:mem=350gb:scratch_local=350gb
#PBS -l walltime=48:00:00


#the paths you might want to change are commented and starting with change
#if you do not need partial results comment our the lines marked with comment out

KSIZE=29 #change to set k
EPS='10' #change to set eps in dbscan
DATA=1 #change to switch between numbered dataset or to '' if unnumbered data
RESALL=/storage/plzen1/home/mertahan/Resall/  # change this to the path where you want to save the results

module add python36-modules-gcc
export PYTHONUSERBASE=/storage/praha1/home/mertahan/.local   #change path
export PATH=$PYTHONUSERBASE/bin:$PATH  
export PYTHONPATH=$PYTHONUSERBASE/lib/python36/site-packages:$PYTHONPATH  


trap 'clean_scratch' TERM EXIT 
cd $SCRATCHDIR || exit 1 


##1. jellyfish
#input scc/data
#output myResults

DATADIR1=/storage/plzen1/home/mertahan/jellyfish/jellyfish-2.2.7 #change the path to installed jellyfish,
#it needs to contain myGenerateCommandsMetacentrum.py
MYDATA=/storage/plzen1/home/mertahan/scc/data$DATA #change the path to the input data
RESDIR1=$RESALL/ResJelly$DATA #change the path to output directory of step 1

cp -r $DATADIR1 . || exit 1

cd jellyfish-2.2.7/

mkdir myResults 

cp -r $MYDATA . || exit 1

python genJellyCom.py --ksize $KSIZE --datadir data$DATA  
chmod 755 my_generate_counts_commands$KSIZE.sh

s=.sh
./my_generate_counts_commands$KSIZE$s

mv myResults myResults$KSIZE
cp -r myResults$KSIZE  $RESDIR1 #comment out for no saving

echo "jellyfish done"

##2. counts table
#input directory Results from jellyfish
#output table$KSIZE

DATADIR2=/storage/plzen1/home/mertahan/mypython3/ #change the path to dictionary.py
RES2=$RESALL/ResultsTable$DATA #change the path to output directory of step 2

mv myResults$KSIZE Data
cp  $DATADIR2/dictionary.py . || exit 1

python dictionary.py --restable table$KSIZE

cp table$KSIZE $RES2 || exit 1 #comment out for no saving

echo "counts done"

##3. dbscan
#input table$KSIZE (from counts table)
#output clusters.txt -> res$KSIZE$EPS
#output sizes, picture

DATADIR3=/storage/plzen1/home/mertahan/mypython3/ #change the path to DBSCAN-clustering2.py
RES3=$RESALL/ResultsClustersDB$DATA 		  #change the path to output directory of step 3

cp $DATADIR3/DBSCAN-clustering2.py . || exit 1

mkdir res$KSIZE"_"$EPS

python DBSCAN-clustering2.py --inptable table$KSIZE

mv PCA_2D_clusters.png res$KSIZE"_"$EPS
cp clusters.txt res$KSIZE"_"$EPS
cp sizes.txt res$KSIZE"_"$EPS
cp plot.pkl res$KSIZE"_"$EPS

cp -r res$KSIZE"_"$EPS $RES3 #comment out for no saving

echo "dbscan done"

##4.readClust
#input clusters.txt, scc/data (copied already)
#output ResultsC$KSIZE$EPS (containing pkl for all fastas)

DATADIR4=/storage/plzen1/home/mertahan/myreadClust/ #change the path to readClustering3.py
RES4=$RESALL/Res$DATA 				    #change the path to output directory of step 4

cp  $DATADIR4/readClustering3.py . || exit 1

python readClustering3.py --ksize $KSIZE --datadir data$DATA  

mv ResultsC ResultsC$KSIZE"_"$EPS
mv Ressig Ressig$KSIZE"_"$EPS
cp -r Ressig$KSIZE"_"$EPS $RES4 				#comment out for no saving
cp -r ResultsC$KSIZE"_"$EPS $RES4 || export CLEAN_SCRATCH=false #comment out for no saving

echo "readClust done"

##5.comparing
 
REFPKL=/storage/plzen1/home/mertahan/myRead2Gene/ResPkl$DATA/Results/ #change the path to Results of reads2Gene2.py 
								      #with processed STAR output
								      #the Results should contain .pkl files

DATADIR5=/storage/plzen1/home/mertahan/mytryComp/		     #change the path to comparing3.py
RES5=$RESALL/Resfin$DATA                                             #change the path to output directory of comparing

cp $DATADIR5/comparing3.py . || exit 2

cp -r $REFPKL . || exit 1

python comparing3.py --myclust ResultsC$KSIZE"_"$EPS --ressig Ressig$KSIZE"_"$EPS --refclust Results

mv results2.txt results$KSIZE"_"$EPS

echo "comparing done"

cp results$KSIZE"_"$EPS $RES5 || export CLEAN_SCRATCH=false
 


